# pylint: disable=invalid-name,line-too-long
import os
from base64 import b64decode
from socket import gethostbyname
from webbrowser import open_new_tab
from requests import get

from colorama import Fore

from .menu import set_readline, tools_cli
from .config import INSTALL_DIR, GITHUB_PATH
from .hosts import get_hosts, add_host
from .utility import Utility


class host2ip(Utility):
    def __init__(self):
        super().__init__(description="Gets IP from host")

    def run(self):
        hosts = get_hosts()
        set_readline(hosts)
        user_host = input("\nEnter a host: ").strip()
        if not user_host in hosts:
            add_host(user_host)
        ip = gethostbyname(user_host)
        print(f"\n{user_host} has the IP of {ip}")


class base64_decode(Utility):
    def __init__(self):
        super().__init__(description="Decodes base64")

    def run(self):
        user_base64 = input("\nEnter base64: ").strip()
        text = b64decode(user_base64)
        print(f"\nDecoded that is: {text}")


class spawn_shell(Utility):
    def __init__(self):
        super().__init__(description="Spawns a local shell")

    def run(self):
        print("Enter `exit` to return to fsociety")
        shell = os.getenv("SHELL", "/bin/bash")
        os.chdir(INSTALL_DIR)
        os.system(shell)


class suggest_tool(Utility):
    def __init__(self):
        super().__init__(description="Suggest a tool or utility")

    def run(self):
        open_new_tab(
            f"https://github.com/{GITHUB_PATH}/issues/new?assignees=&labels=tool&template=---tool-request.md&title="
        )


class print_contributors(Utility):
    def __init__(self):
        super().__init__(description="Prints the usernames of our devs")

    def run(self):
        print(Fore.RED + """
    8888b.  888888 Yb    dP .dP"Y8 
    8I  Yb 88__    Yb  dP  `Ybo." 
    8I  dY 88""     YbdP   o.`Y8b 
    8888Y"  888888    YP    8bodP'
    """)
        response = get(
            f"https://api.github.com/repos/{GITHUB_PATH}/contributors")
        contributors = response.json()
        for contributor in sorted(contributors,
                                  key=lambda c: c['contributions'],
                                  reverse=True):
            username = contributor.get("login")
            print(f" {username} ".center(30, "-"))
        print(Fore.RESET)


__tools__ = [
    tool() for tool in
    [host2ip, base64_decode, spawn_shell, suggest_tool, print_contributors]
]


def cli():
    tools_cli(__name__, __tools__)



from abc import ABCMeta, abstractmethod


class Utility(metaclass=ABCMeta):
    def __init__(self, description=None):
        self.description = description

    def __str__(self):
        return self.__class__.__name__

    @abstractmethod
    def run(self):
        pass



import os

from fsociety.core.repo import GitHubRepo


class HashBusterRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="s0md3v/Hash-Buster",
            install=None,
            description="Why crack hashes when you can bust them?")

    def run(self):
        os.chdir(self.full_path)
        user_hash = input("\nEnter a hash: ").strip()
        return os.system(f"python3 hash.py -s {user_hash}")


hash_buster = HashBusterRepo()



# pylint: disable=line-too-long
import os
from shutil import which

from fsociety.core.repo import GitHubRepo


class BettercapRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="bettercap/bettercap",
            install={
                "linux":
                "sudo apt install golang git build-essential libpcap-dev libusb-1.0-0-dev libnetfilter-queue-dev; go get -u github.com/bettercap/bettercap",
                "brew": "install bettercap"
            },
            description="Swiss army knife for network attacks and monitoring")

    def installed(self):
        return which("bettercap")

    def install(self):
        super().install(clone=False)

    def run(self):
        print("Please note that bettercap must be run with sudo")
        return os.system("sudo bettercap")


bettercap = BettercapRepo()



# Core
from fsociety.core.menu import tools_cli

from .nmap import nmap
from .bettercap import bettercap

__tools__ = [str(tool) for tool in [nmap, bettercap]]


def cli():
    tools_cli(__name__, __tools__)



import os

from fsociety.core.repo import GitHubRepo


class CuppRepo(GitHubRepo):
    def __init__(self):
        super().__init__(path="Mebus/cupp",
                         install=None,
                         description="Common User Passwords Profiler")

    def run(self):
        os.chdir(self.full_path)
        return os.system("python3 cupp.py -i")


cupp = CuppRepo()



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import confirm


class PhotonRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="s0md3v/Photon",
            install={"pip": "requirements.txt"},
            description="Incredibly fast crawler designed for OSINT")

    def run(self):
        os.chdir(self.full_path)
        user_url = input("\nEnter a url to scan: ").strip()
        args = list()
        if confirm("Do you want to clone the site?"):
            args.append("--clone")
        if confirm("Do you want to use wayback?"):
            args.append("--wayback")
        args_str = " ".join(args)
        return os.system(f"python3 photon.py --url {user_url} {args_str}")


photon = PhotonRepo()



# Core
from fsociety.core.menu import tools_cli

from .xsstrike import xsstrike
from .photon import photon

__tools__ = [str(tool) for tool in [xsstrike, photon]]


def cli():
    tools_cli(__name__, __tools__)



import os
from abc import ABCMeta, abstractmethod
from shutil import which

from fsociety.core.config import INSTALL_DIR, get_config
from fsociety.core.menu import confirm

config = get_config()


class InstallError(Exception):
    pass


class CloneError(Exception):
    pass


class GitHubRepo(metaclass=ABCMeta):
    def __init__(self,
                 path="fsociety-team/fsociety",
                 install="pip install -e .",
                 description=None):
        self.path = path
        self.name = self.path.split("/")[-1]
        self.install_options = install
        self.full_path = os.path.join(INSTALL_DIR, self.name)
        self.description = description

    def __str__(self):
        return self.name.lower().replace("-", "_")

    def clone(self):
        if os.path.exists(self.full_path):
            os.chdir(self.full_path)
            os.system("git pull")
            return self.full_path
        url = f"https://github.com/{self.path}"
        if config.getboolean("fsociety", "ssh_clone"):
            url = f"git@github.com:{self.path}.git"
        os.system(f"git clone {url} {self.full_path}")
        if not os.path.exists(self.full_path):
            raise CloneError(f"{self.full_path} not found")
        return self.full_path

    def install(self, no_confirm=False, clone=True):
        if no_confirm or not confirm(
                f"\nDo you want to install https://github.com/{self.path}?"):
            print("Cancelled")
            return
        if clone:
            self.clone()
        if self.install_options:
            os.chdir(self.full_path)
            install = self.install_options

            if isinstance(install, dict):
                if "pip" in install.keys():
                    packages = install.get("pip")
                    if isinstance(packages, list):
                        message = f"Do you want to install the following packages? {packages}"
                        packages_str = " ".join(packages)
                        command = f"pip install {packages_str}"
                    elif isinstance(packages, str):
                        requirements_txt = os.path.join(
                            self.full_path, "requirements.txt")
                        message = f"Do you want to install the packages in {requirements_txt}?"
                        command = f"pip install -r {requirements_txt}"

                    if not confirm(message):
                        raise InstallError
                elif config.get("fsociety",
                                "os") == "macos" and "brew" in install.keys(
                                ) and which("brew"):
                    brew_opts = install.get("brew")
                    command = f"brew {brew_opts}"
                elif "linux" in install.keys() or "windows" in install.keys(
                ) or "macs" in install.keys():
                    command = install.get(config.get("fsociety", "os"),
                                          install.get("linux"))
            else:
                command = install

            print()
            os.system(command)

    def installed(self):
        return os.path.exists(self.full_path)

    @abstractmethod
    def run(self):
        pass


class Gist(GitHubRepo):
    def clone(self):
        if os.path.exists(self.full_path):
            os.chdir(self.full_path)
            os.system("git pull")
            return self.full_path
        url = f"https://gist.github.com/{self.path}"
        if config.getboolean("fsociety", "ssh_clone"):
            url = f"git@gist.github.com:{self.path}.git"
        os.system(f"git clone {url} {self.full_path}")
        if not os.path.exists(self.full_path):
            raise CloneError(f"{self.full_path} not found")
        return self.full_path

    @abstractmethod
    def run(self):
        pass



import os

from fsociety.core.repo import GitHubRepo


class CuteitRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="D4Vinci/Cuteit",
            install=None,
            description="IP obfuscator made to make a malicious ip a bit cuter"
        )

    def run(self):
        os.chdir(self.full_path)
        user_ip = input("\nEnter a ip: ").strip()
        return os.system(f"python3 Cuteit.py {user_ip}")


cuteit = CuteitRepo()



# pylint: disable=unused-import,broad-except
import os
import shutil

from colorama import Fore, Back, Style

from fsociety.core.config import INSTALL_DIR

BACK_COMMANDS = ["exit", "back", "return"]


class CommandCompleter():
    def __init__(self, options):
        self.options = sorted(options)
        self.matches = list()

    def complete(self, text, state):
        response = None
        if state == 0:
            if text:
                self.matches = [
                    s for s in self.options if s and s.startswith(text)
                ]
            else:
                self.matches = self.options[:]
        try:
            response = self.matches[state]
        except IndexError:
            response = None
        return response


def set_readline(items):
    try:
        import readline
    except ImportError:
        pass
    else:
        import rlcompleter
        if isinstance(items, list):
            readline.set_completer(CommandCompleter(items).complete)
        elif isinstance(items, dict):
            readline.set_completer(CommandCompleter(items.keys()).complete)
        else:
            readline.set_completer(CommandCompleter(list(items)).complete)
        readline.parse_and_bind("tab: complete")


def clear_screen():
    os.system('cls' if os.name == 'nt' else 'clear')


def format_menu_item(item):
    string = str(item)
    if hasattr(item, "description") and item.description:
        string += f" - {item.description}"
    return f"{Back.WHITE}{Fore.BLACK}{string}{Style.RESET_ALL}"


def format_tools(tools):
    etc = False
    if len(tools) > 3:
        tools = tools[:3]
        etc = True
    res = "".join([f"\n\t{str(tool)}" for tool in tools])
    if etc:
        res += "\n\t..."
    return res


def module_name(module):
    return module.__name__.split(".")[-1]


def prompt(path="", base_path="~"):
    return f"{Fore.RED}fsociety {os.path.join(base_path, path, '')}#{Fore.WHITE} "


def input_wait():
    input("\nPress [ENTER] to continue... ")


def tools_cli(name, tools):
    tools_dict = dict()
    for tool in tools:
        tools_dict[str(tool)] = tool
        print(f"{format_menu_item(tool)}\n")
    print(f"{format_menu_item(str('back'))}\n")
    set_readline(list(tools_dict.keys()) + BACK_COMMANDS)
    selected_tool = input(prompt(name.split(".")[-2])).strip()
    if not selected_tool in tools_dict.keys():
        if selected_tool in BACK_COMMANDS:
            return
        print(f"{Fore.YELLOW}Invalid Command{Fore.RESET}")
        return
    tool = tools_dict.get(selected_tool)
    if hasattr(tool, "install") and not tool.installed():
        tool.install()
    try:
        response = tool.run()
        if response and response > 0:
            raise Exception
    except KeyboardInterrupt:
        return
    except Exception as error:
        print(f"{Fore.RED + selected_tool} failed{Fore.RESET}")
        print(str(error))
        if hasattr(tool, "install") and confirm("Do you want to reinstall?"):
            os.chdir(INSTALL_DIR)
            shutil.rmtree(tool.full_path)
            tool.install()
    input_wait()


def confirm(message="Do you want to?"):
    response = input(f"{message} (y/n): ").lower()
    if response:
        return response[0] == "y"
    return False



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline
from fsociety.core.usernames import get_usernames, add_username


class SherlockRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="sherlock-project/sherlock",
            install={"pip": "requirements.txt"},
            description=
            "Hunt down social media accounts by username across social networks"
        )

    def run(self):
        os.chdir(self.full_path)
        usernames = get_usernames()
        set_readline(usernames)
        user_usernames = input("\nEnter one or more usernames: ").strip()
        for username in user_usernames.split():
            if not username in usernames:
                add_username(username)
        return os.system(f"python3 sherlock {user_usernames} -r")


sherlock = SherlockRepo()



import os.path

from fsociety.core.config import INSTALL_DIR, get_config

config = get_config()

full_path = os.path.join(INSTALL_DIR, config.get("fsociety", "host_file"))


class InvalidHost(Exception):
    pass


def get_hosts():
    try:
        with open(full_path, "r") as hostfile:
            return [host.strip() for host in hostfile]
    except FileNotFoundError:
        return list()


def add_host(host):
    if not host:
        raise ValueError
    with open(full_path, "a") as hostfile:
        hostfile.write(f"\n{host}")



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline
from fsociety.core.hosts import get_hosts, add_host, InvalidHost


class ChangemeRepo(GitHubRepo):
    def __init__(self):
        super().__init__(path="ztgrace/changeme",
                         install={"pip": "requirements.txt"},
                         description="A default credential scanner")

    def run(self):
        os.chdir(self.full_path)
        hosts = get_hosts()
        set_readline(hosts)
        user_host = input("\nEnter a host: ").strip()
        if not user_host:
            raise InvalidHost
        if user_host not in hosts:
            add_host(user_host)
        return os.system(f"python3 changeme.py {user_host}")


changeme = ChangemeRepo()



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline, confirm


class HydrareconRepo(GitHubRepo):
    def __init__(self):
        super().__init__(path="aufzayed/HydraRecon",
                         install={"pip": "requirements.txt"},
                         description="Simple recon tool")

    def run(self):
        os.chdir(self.full_path)
        set_readline([])
        user_domain = input("\nEnter a domain to scan: ").strip()
        arg = "--basic"
        if confirm("\nDo you want to crawl? [default=No]"):
            arg = "--crawl"
        return os.system(f"python3 hydrarecon.py {arg} -d {user_domain}")


hydrarecon = HydrareconRepo()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# pylint: disable=exec-used,broad-except,inconsistent-return-statements,invalid-name

import argparse
import platform
from random import choice

import colorama
from colorama import Fore, Style

# Core
from fsociety.core.menu import (set_readline, format_menu_item, format_tools,
                                module_name, prompt, clear_screen)
from fsociety.core.config import get_config, write_config, CONFIG_FILE
import fsociety.core.utilities
import fsociety.information_gathering
import fsociety.passwords
import fsociety.web_apps
import fsociety.obfuscation
import fsociety.networking

# Config
config = get_config()

# Menu
TERMS = Fore.YELLOW + """
I shall not use fsociety to:
(i) upload or otherwise transmit, display or distribute any
content that infringes any trademark, trade secret, copyright
or other proprietary or intellectual property rights of any
person; (ii) upload or otherwise transmit any material that contains
software viruses or any other computer code, files or programs
designed to interrupt, destroy or limit the functionality of any
computer software or hardware or telecommunications equipment;
""" + Fore.RESET
BANNER1 = Fore.RED + """
    ____                _      __       
   / __/________  _____(_)__  / /___  __
  / /_/ ___/ __ \/ ___/ / _ \/ __/ / / /
 / __(__  ) /_/ / /__/ /  __/ /_/ /_/ / 
/_/ /____/\____/\___/_/\___/\__/\__, /  
                               /____/   
"""
BANNER2 = Fore.YELLOW + """
  __                _      _         
 / _|___  ___   ___(_) ___| |_ _   _ 
| |_/ __|/ _ \ / __| |/ _ \ __| | | |
|  _\__ \ (_) | (__| |  __/ |_| |_| |
|_| |___/\___/ \___|_|\___|\__|\__, |
                               |___/ 
"""
BANNER3 = Fore.GREEN + """
 .-.                     .      
 |                o     _|_     
-|-.--. .-.  .-.  .  .-. |  .  .
 | `--.(   )(     | (.-' |  |  |
 ' `--' `-'  `-'-' `-`--'`-'`--|
                               ;
                            `-' 
"""
BANNER4 = Fore.MAGENTA + """
 ,__                                 .           
 /  `   ____   __.    ___  `   ___  _/_   ,    . 
 |__   (     .'   \ .'   ` | .'   `  |    |    ` 
 |     `--.  |    | |      | |----'  |    |    | 
 |    \___.'  `._.'  `._.' / `.___,  \__/  `---|.
 /                                         \___/ 
"""
BANNERS = [BANNER1, BANNER2, BANNER3, BANNER4]
MENU_ITEMS = [
    fsociety.information_gathering, fsociety.networking, fsociety.web_apps,
    fsociety.passwords, fsociety.obfuscation, fsociety.core.utilities
]
BUILTIN_FUNCTIONS = {
    "exit": lambda: exec('raise KeyboardInterrupt'),
}
items = dict()


def menuitems():
    items_str = str()
    for value in MENU_ITEMS:
        name = module_name(value)
        tools = format_tools(value.__tools__)
        items_str += f"{format_menu_item(name)} {tools}\n\n"
    for key in BUILTIN_FUNCTIONS:
        items_str += f"{format_menu_item(key)}\n\n"
    return items_str


def agreement():
    while not config.getboolean("fsociety", "agreement"):
        clear_screen()
        print(TERMS)
        agree = input(
            "You must agree to our terms and conditions first (Y/n) ")
        if agree.lower()[0] == "y":
            config.set('fsociety', 'agreement', 'true')


for item in MENU_ITEMS:
    items[module_name(item)] = item

commands = list(items.keys()) + list(BUILTIN_FUNCTIONS.keys())


def mainloop():
    agreement()
    print(choice(BANNERS) + Fore.RESET)
    print(menuitems())
    selected_command = input(prompt()).strip()
    if not selected_command or (not selected_command in commands):
        print(f"{Fore.YELLOW}Invalid Command{Fore.RESET}")
        return
    if selected_command in BUILTIN_FUNCTIONS.keys():
        func = BUILTIN_FUNCTIONS.get(selected_command)
        return func()
    print(Style.RESET_ALL)
    try:
        func = items[selected_command].cli
        return func()
    except Exception as error:
        print(str(error))
    return


def info():
    data = dict()
    # Config File
    with open(CONFIG_FILE) as file:
        data["Config File"] = file.read().strip()
    data["Python Version"] = platform.python_version()
    data["Platform"] = platform.platform()
    os = config.get("fsociety", "os")
    if os == "macos":
        data["macOS"] = platform.mac_ver()[0]
    elif os == "windows":
        data["Windows"] = platform.win32_ver()[0]

    for key, value in data.items():
        print(f"# {key}")
        print(value)
        print()


def interactive():
    colorama.init()
    try:
        while True:
            set_readline(commands)
            mainloop()
    except KeyboardInterrupt:
        print("\nExitting...")
        write_config(config)
        exit(0)


def main():
    parser = argparse.ArgumentParser(
        description='A Penetration Testing Framework')
    parser.add_argument('-i',
                        '--info',
                        action='store_true',
                        help='gets fsociety info')
    parser.add_argument('-s',
                        '--suggest',
                        action='store_true',
                        help='suggest a tool')
    # parser.add_argument('-w', '--web', action='store_true', help='start web ui')
    # parser.add_argument('-t', '--tool', help='run tool')

    args = parser.parse_args()

    if args.info:
        info()
    elif args.suggest:
        fsociety.core.utilities.suggest_tool()
    # elif args.tool:
    #     print("TODO: Run tool by name")
    # elif args.web:
    #     print("TODO: Webserver Here")
    else:
        interactive()


if __name__ == "__main__":
    main()



import os.path
from sys import platform
from pathlib import Path
from configparser import RawConfigParser, NoOptionError

from fsociety.__version__ import __version__

CURRENT_PLATFORM = platform
if platform in ["win32", "cygwin"]:
    CURRENT_PLATFORM = "windows"
elif platform.startswith("darwin"):
    CURRENT_PLATFORM = "macos"
elif platform.startswith("linux") or platform.startswith("freebsd"):
    CURRENT_PLATFORM = "linux"

INSTALL_DIR = os.path.join(str(Path.home()), ".fsociety")
CONFIG_FILE = os.path.join(INSTALL_DIR, "fsociety.cfg")
GITHUB_PATH = "fsociety-team/fsociety"

DEFAULT_CONFIG = {
    "version": __version__,
    "agreement": "false",
    "ssh_clone": "false",
    "os": CURRENT_PLATFORM,
    "host_file": "hosts.txt",
    "usernames_file": "usernames.txt"
}


def get_config():
    config = RawConfigParser()
    if not os.path.exists(INSTALL_DIR):
        os.mkdir(INSTALL_DIR)
    if not os.path.exists(CONFIG_FILE):
        config["fsociety"] = DEFAULT_CONFIG
        with open(CONFIG_FILE, "w") as configfile:
            config.write(configfile)
    config.read(CONFIG_FILE)
    check_config(config)
    if config.get("fsociety", "version") != __version__:
        config.set("fsociety", "version", __version__)
        write_config(config)
    return config


def write_config(config):
    with open(CONFIG_FILE, "w") as configfile:
        config.write(configfile)


def check_config(config):
    for key in DEFAULT_CONFIG:
        try:
            config.get("fsociety", key)
        except NoOptionError:
            config.set("fsociety", key, DEFAULT_CONFIG.get(key))
    write_config(config)



# Core
from fsociety.core.menu import tools_cli

from .cupp import cupp
from .cr3dov3r import cr3dov3r
from .hash_buster import hash_buster
from .changeme import changeme

__tools__ = [str(tool) for tool in [cupp, cr3dov3r, hash_buster, changeme]]


def cli():
    tools_cli(__name__, __tools__)



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import confirm


class XsstrikeRepo(GitHubRepo):
    def __init__(self):
        super().__init__(path="s0md3v/XSStrike",
                         install={"pip": "requirements.txt"},
                         description="Advanced XSS Detection Suite")

    def run(self):
        os.chdir(self.full_path)
        user_url = input("\nEnter a url to scan: ").strip()
        args = list()
        if confirm("Do you want to crawl?"):
            args.append("--crawl")
        if confirm("Do you want to find hidden parameters?"):
            args.append("--params")
        args_str = " ".join(args)
        return os.system(f"python3 xsstrike.py --url {user_url} {args_str}")


xsstrike = XsstrikeRepo()



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import set_readline


class Sublist3rRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="aboul3la/Sublist3r",
            install={"pip": "requirements.txt"},
            description=
            "Fast subdomains enumeration tool for penetration testers")

    def run(self):
        os.chdir(self.full_path)
        set_readline([])
        user_domain = input("\nEnter a domain to enumerate: ").strip()
        return os.system(f"python3 sublist3r.py -v -d {user_domain}")


sublist3r = Sublist3rRepo()



import os

from fsociety.core.repo import GitHubRepo
from fsociety.core.menu import confirm


class SqlmapRepo(GitHubRepo):
    def __init__(self):
        super().__init__(
            path="sqlmapproject/sqlmap",
            install=None,
            description="Automatic SQL injection and database takeover tool")

    def run(self):
        os.chdir(self.full_path)
        user_url = input("\nEnter a url to scan: ").strip()
        user_args = str()
        if confirm("\nDo you want to add any extra args?"):
            os.system("python3 sqlmap.py --help")
            user_args = input("\nEnter any extra args: ").strip()
        return os.system(f"python3 sqlmap.py -u {user_url} {user_args}")


sqlmap = SqlmapRepo()
